      Program ba_gm_vs_m

* Program to compute ground motions vs M from
* BA provisional GMPE

* Uses control file:
*! Control file for program ba_gm_vs_m.for
*!Header to add to output file (no "!" at beginning!)
* [blank]
*!file containing regression coefficients:
*  c:\peer_nga\teamx\ba_02apr07_usnr_add_t10_e3.out
*!stem name of output file:
* ba_gm_vs_m_r0.1_10_30_v30_760_mech_r
*!mechanism (u,s,n,r):
* r
*!v30:
* 255.0
*!mstart, dmag, mstop:
* 4.0 0.25 8.0
*!nrjb
* 3
*!rjb:
* 0.1
* 10.0
* 30.0
*!no. of periods for output:
* 6 
*!periods (<0 for pgv, 0.0 for pga):
* -1.0 0.0 0.1 0.2 1.0 3.0
   
* Dates: 02/24/06 - written by D.M. Boore, based on ba_gm_vs_r
*        03/01/06 - Modify for new coefficient file format (one file
*                   for all mechanisms, including unspecified).
*        07/03/06 - Allow for cubic smoothing of nonlinear amps.
*        10/10/06 - Allow for more periods.
*        02/03/07 - Allow for more periods.
*        02/03/07 - Delete path input (not needed)
*        03/26/07 - Read over header lines (preceded by "!") in the 
*                   regression coefficient file.
*        03/29/07 - Correct error in constructing name of event term output file
*                   when T >= 10.0
*        10/30/07 - Skip calculation for nomral fault at T=10 s if
*                   e3 = 999.99. 
*        10/30/07 - Remove some printing to screen,as it hides the printing of
*                   error messages (such as normal fault, T = 10, 
*                   when e3 = 999.99)
*        03/13/08 - pga4nl from BA GMPEs, rather than the equation used to
*                   correct the data before doing the regressions.

   
      real per(120)    
      
      character buf*80, f_ctl*80, f_file*80, 
     :          f_coeff*120, f_out_stem*80,
     :          header4output*80, f_out*80, mech*10,
     :          format_out*80
     
      
      character coeff_head(10)*3
      
      character cmnts2skip(200)*79
      
      logical file_exist 
      
      real per_stage1(120), per_stage2(120)
      
      real per_gmpe(120), e_gmpe(8,120), amh_gmpe(120), 
     :     c_gmpe(4,120), amref_gmpe(120), rref_gmpe(120), h_gmpe(120), 
     :     blin(120), vref(120), b1(120), b2(120), v1(120), v2(120),
     :     a1(120), pga_low(120), a2(120),
     :     sig1_gmpe(120), 
     :     sig2_gmpe_u(120), sigt_gmpe_u(120),
     :     sig2_gmpe_m(120), sigt_gmpe_m(120),
     :     rdum(120)
        
      real e_pga4nl(8), amh_pga4nl, 
     :     c_pga4nl(4), amref_pga4nl, rref_pga4nl, h_pga4nl, 
     :     sigt_pga4nl
        
      real m(200), logy_pred, mstart, mstop, dmag, rjb(20)
      real y(200), sigy(200) 
      
      integer nmag
 
      pi = 4.0*atan(1.0)
      twopi = 2.0 * pi
      
      ncoeff_max = 5
      nper_max = 120
      
      ncoeff_stage1 = 4      
      ncoeff_stage2 = 8

*DEBUG
      call get_lun(nu_debug)
      open(unit=nu_debug,file='debug.out',status='unknown')
*DEBUG

      do j = 1, nper_max
        sig1_gmpe(j) = 0.0
        sig2_gmpe_u(j) = 0.0
        sigt_gmpe_u(j) = 0.0
        sig2_gmpe_m(j) = 0.0
        sigt_gmpe_m(j) = 0.0
      end do

      do i = 1, ncoeff_stage1
        c_pga4nl(i) = 0.0
        do j = 1, nper_max
          c_gmpe(i,j) = 0.0
        end do
      end do
      
      do i = 1, ncoeff_stage2
        e_pga4nl(i) = 0.0
        do j = 1, nper_max
          e_gmpe(i,j) = 0.0
        end do
      end do
      
      file_exist = .false.
      do while (.not. file_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = ba_gm_vs_m.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'ba_gm_vs_m.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=file_exist)
        if (.not. file_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do
      
      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)  
      header4output = ' '
      read(nu_ctl,'(a)') header4output
      call trim_c(header4output,nc_h4out)
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)      
      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf,nc_buf)
      f_coeff  = ' '
      f_coeff  = buf(1:nc_buf)
      call trim_c(f_coeff, nc_f_coeff)
 
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)      
      f_out_stem = ' '
      read(nu_ctl,'(a)') f_out_stem
      call trim_c(f_out_stem,nc_f_out_stem)
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)      
      mech = ' '
      read(nu_ctl,'(a)') mech
      call trim_c(mech,nc_mech)
      call upstr(mech)

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)      
      read(nu_ctl,*) v30
 
 
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)      
      read(nu_ctl,*) mstart, dmag, mstop
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)      
      read(nu_ctl,*) nrjb
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip) 
      do i = 1, nrjb
        read(nu_ctl,*) rjb(i)
      end do
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)      
      read(nu_ctl,*) npout
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) (per(i),i=1,npout)
	
      close(nu_ctl)

      nmag = int((mstop - mstart)/dmag + 1.1)
      
      if (nmag .lt. 1) then
        nmag = 1
      end if
      
      do i = 1, nmag
        m(i) = mstart + float(i-1)*dmag
      end do
      
* Open coefficient file:

      file_exist = .false.
      inquire(file=f_coeff(1:nc_f_coeff), exist=file_exist)
      if (.not. file_exist) then
        print *, ' ******* COEFF FILE '//f_coeff(1:nc_f_coeff)//
     :                 ' DOES NOT EXIST ******* '
        print *, ' QUITTING!!!!'
        close(nu_ctl)
        stop
      end if
 
      print *,' f_coeff  = '//
     :          f_coeff(1:nc_f_coeff)
      
      call get_lun(nu_coeff)
      open(unit=nu_coeff,file=f_coeff(1:nc_f_coeff),
     :     status='unknown')
     
      call skipcmnt(nu_coeff,cmnts2skip,nc_cmnts2skip)      
      call skip(nu_coeff,1)  ! The column header row is NOT preceded with '!'
                             ! because might want to open the coefficient file
                             ! in Excel.
           
           
* read coefficients for pga4nl:

      read(nu_coeff,*) 
     :   per_pga4nl, (e_pga4nl(j), j = 1, ncoeff_stage2),
     :   amh_pga4nl, (c_pga4nl(j), j = 1, ncoeff_stage1), 
     :   amref_pga4nl, rref_pga4nl, h_pga4nl 

 
* read coefficients for regression coefficients:

      i = 0
3001  continue

      i = i + 1
      
      read(nu_coeff,*,end=3999) 
     :   per_gmpe(i), (e_gmpe(j,i), j = 1, ncoeff_stage2),
     :   amh_gmpe(i), (c_gmpe(j,i), j = 1, ncoeff_stage1), 
     :   amref_gmpe(i), rref_gmpe(i), h_gmpe(i), 
     :   blin(i), vref(i), b1(i), b2(i), v1(i), v2(i), 
     :   a1(i), pga_low(i), a2(i),
     :   sig1_gmpe(i), 
     :   sig2_gmpe_u(i), sigt_gmpe_u(i),
     :   sig2_gmpe_m(i), sigt_gmpe_m(i)
      
       
      goto 3001
3999  continue
      
      close(nu_coeff)
      
      nper_gmpe = i-1
      
!**** Coefficients for pga4nl from GMPE (modification on 13 March 2008):

!    Find index for pga (period = 0.0):

      do i = 1, nper_gmpe
        if (per_gmpe(i) .eq. 0.0) then
          indx_pga = i
        end if
      end do
      
!    Store coefficients for pga4nl:

      do j = 1, ncoeff_stage2
        e_pga4nl(j) = e_gmpe(j,indx_pga)
      end do
      
      do j = 1, ncoeff_stage1
        c_pga4nl(j) = c_gmpe(j,indx_pga)
      end do
      
      amh_pga4nl = amh_gmpe(indx_pga) 
      amref_pga4nl = amref_gmpe(indx_pga)
      rref_pga4nl = rref_gmpe(indx_pga)
      h_pga4nl = h_gmpe(indx_pga)
!**** Coefficients for pga4nl from GMPE (modification on 13 March 2008):

        
* LOOP OVER PERIODS:

      do iper = 1, npout

        do i = 1, nper_gmpe
          if(per(iper) .eq. per_gmpe(i)) then
            index_per = i
            go to 3111
          end if
        end do
      
        print *,' per not found in gmpe coefficient file,'//
     :        ' returning for another period'
        print *,' per = ', per(i)
        
        go to 3000
3111    continue  

* Check for normal fault, T = 10:
        if (mech .eq. 'N' .and. per(iper) .eq. 10.0) then
          if (e_gmpe(3,index_per) .eq. 999.99) then
            print *, ' Normal fault motion for'//
     :               ' T = 10 s is undefined, skipping'//
     :               ' to next set of predictor variables'
            go to 3000
          end if
        end if
        
 
        f_out = ' '
        f_out = f_out_stem(1:nc_f_out_stem)
        
        if(per(iper) .lt. 0.0) then
          write(
     :      f_out(nc_f_out_stem+1:nc_f_out_stem+4),'(a4)') 
     :                  '.pgv'
        else if(per(iper) .eq. 0.0) then
          write(
     :      f_out(nc_f_out_stem+1:nc_f_out_stem+4),'(a4)') 
     :                '.pga'
        else if(per(iper) .ge. 10.0) then
          write(
     :      f_out(nc_f_out_stem+1:nc_f_out_stem+8),'(a2,f6.3)') 
     :                '.t', per(iper)
        else
          write(
     :      f_out(nc_f_out_stem+1:nc_f_out_stem+7),'(a2,f5.3)') 
     :                '.t', per(iper)
        end if
        
        call trim_c(f_out, nc_f_out)
        f_out(nc_f_out+1:nc_f_out+4) = '.out'
        call trim_c(f_out, nc_f_out)
      
        call get_lun(nu_out)
        open(unit=nu_out,file=f_out(1:nc_f_out),status='unknown')

      
        format_out = ' '
        format_out = "(10x,'M',"//    
     :      "zz(5x,'y',f5.1, 1x,'y/sig',f5.1, 1x,'y*sig',f5.1))"
        call trim_c(format_out, nc_format_out)
        ind_z = index(format_out(1:nc_format_out),'z')

        write(format_out(ind_z:ind_z+1),'(i2.2)') nrjb

        write(nu_out, format_out(1:nc_format_out)) 
     :         (rjb(i), rjb(i), rjb(i), i=1, nrjb)

        do j = 1, nmag
          do i = 1, nrjb 
          
!            print *,' per, m, rjb = ', per(iper), m(j), rjb(i)

            if (mech(1:1) .eq. 'U') then
              sigt = sigt_gmpe_u(index_per)
            else
              sigt = sigt_gmpe_m(index_per)
            end if

            call ba_gm_sub4y(
     :        per(iper), m(j),rjb(i),r,
     :        mech(1:1),v30,y(i),sigy(i),
     :        e_gmpe(1,index_per), amh_gmpe(index_per), 
     :        c_gmpe(1,index_per), amref_gmpe(index_per),
     :        rref_gmpe(index_per), h_gmpe(index_per), 
     :        vref(index_per), 
     :        a1(index_per), pga_low(index_per), a2(index_per),
     :        sigt,
     :        e_pga4nl, amh_pga4nl, 
     :        c_pga4nl, amref_pga4nl,
     :        rref_pga4nl, h_pga4nl,
     :        pga4nl, fm_pga4nl, fd_pga4nl, fs_pga4nl,
     :        bnl, amp_lin, amp_nl, amp_total,
     :        fm_gmpe, fd_gmpe, fs_gmpe)

          end do

* Convert from g to cgs units:

         if (per(iper) .ge. 0.0) then
           do i = 1, nrjb
             y(i) = 981.0*y(i)
           end do
         end if

          write(nu_out,'(1p,60(1x,e10.3))')
     :      m(j), (y(i), y(i)/sigy(i), y(i)*sigy(i), i = 1, nrjb)
     
        end do  ! END LOOP OVER DISTANCE
      
        close(nu_out)
       
3000    continue
      end do  ! END LOOP OVER PERIOD
      
*DEBUG
      close(nu_debug)
*DEBUG
       
      stop
      end

      include 'ba_gm_sub4y.for'
      include '\forprogs\skipcmnt.for'
        
      include '\forprogs\get_lun.for'
      include '\forprogs\lin_interp.for'
      include '\forprogs\locate.for'
      include '\forprogs\skip.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'

 
